/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.FarmID;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

/**
 * registry of managed farm IDs and their project locations.
 * 
 * @author johan
 */
public class FarmProjectRegistry {
    private static final String REGISTRY_FILE = "managedFarms.ser";
    private static final Logger LOGGER = Logger.getLogger(FarmProjectRegistry.class.getName());    
    
    private static FarmProjectRegistry instance;
    
    private Map<FarmID,String> managedFarms;
    
    private FarmProjectRegistry() {
        load();
    }
    
    static synchronized FarmProjectRegistry getInstance() {
        if (instance == null) {
            instance = new FarmProjectRegistry();
        }
        return instance;
    }
    
    private void load() {
        FileObject fo = getRegistryFile(false);
        if (fo == null) {
            // initialize:
            managedFarms = new HashMap<>();
            save();
        } else {
            // read existing file:
            try (ObjectInputStream ois = new ObjectInputStream(fo.getInputStream())) {
                managedFarms = (Map<FarmID,String>) ois.readObject();
            } catch (IOException | ClassNotFoundException ex) {
                LOGGER.log(Level.SEVERE, "could not load registry from {0} with error: {1}", new Object[]{fo.getPath(), ex.getMessage()});
            }
        }
    }
    
    private synchronized void save() {
        FileObject fo = getRegistryFile(true);
        try (ObjectOutputStream oos = new ObjectOutputStream(fo.getOutputStream())) {
            oos.writeObject(managedFarms);
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "could not save registry to {0} with error: {1}", new Object[]{fo.getPath(), ex.getMessage()});
        }
    }

    private FileObject getRegistryFile(boolean create) {
        FileObject fo = getRegistryFolder().getFileObject(REGISTRY_FILE);
        if (fo == null && create) {
            try {
                fo = getRegistryFolder().createData(REGISTRY_FILE);
            } catch (IOException ex) {
                LOGGER.log(Level.SEVERE, "could not create registry file with error: {1}", new Object[]{ex.getMessage()});
            }
        }
        return fo;
    }
    
    FileObject getRegistryFolder() {
        return FileUtil.getConfigRoot();
    }
    
    void add(FarmProject project) {
        LOGGER.log(Level.FINEST, "adding farm project {0} to registry", new Object[]{project.getFarm().getName()});
        managedFarms.put(project.getFarmID(), project.getProjectDirectory().getPath());
        save();
    }
    
    void remove(FarmProject project) {
        LOGGER.log(Level.FINEST, "removing farm project {0} from registry", new Object[]{project.getFarm().getName()});
        managedFarms.remove(project.getFarmID());
        save();
    }
    
    boolean contains(FarmProject project) {
        return managedFarms.containsKey(project.getFarmID());
    }
    boolean contains(EnterpriseIdProvider eip) {
        return managedFarms.containsKey(new FarmID(eip));
    }
    
    String getRegisteredPath(FarmProject project) {
        return managedFarms.get(project.getFarmID());
    }

}
