/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmDetailsController;
import nl.cloudfarming.client.model.Address;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.IssuingAgency;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */
@NbBundle.Messages({
    "FarmWizardDetailsPanel invalid message=There are some errors please correct them",
    "FarmWizardDetailsPanel invalid enterprise id=The combination issuing agency / registration number must be unique",
})
public class FarmWizardDetailsPanel extends FarmDetailsController<FarmWizardDetailsVisual> implements WizardDescriptor.Panel,
        WizardDescriptor.FinishablePanel {

    private WizardDescriptor wizardDescriptor;
    private final Set<ChangeListener> listeners = new HashSet<>(1); // or can use ChangeSupport in NB 6.0

    public FarmWizardDetailsPanel() {
        super(new FarmWizardDetailsVisual(new Address()));
        farmDetailPanel.getFarmIssuingComboBox().addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                farmDetailFieldsUpdated();
            }
        });
    }

    @Override
    public Component getComponent() {
        return super.getPanel();
    }

    @Override
    public HelpCtx getHelp() {
        return new HelpCtx(FarmWizardDetailsPanel.class);
    }

    @Override
    public void farmDetailFieldsUpdated() {
        this.fireChangeEvent();
    }

    @Override
    public boolean isValid() {

        boolean valid = super.validate();
        String errorMsg = "";
        
        // check for unique URI:
        if (valid && !isUniqueEnterpriseId()) {
            valid = false;
            farmDetailPanel.getRegistrationNumberRow().setInvalid(Bundle.FarmWizardDetailsPanel_invalid_enterprise_id());
        }
        
        if (!valid) {
            errorMsg = Bundle.FarmWizardDetailsPanel_invalid_message();
        }
        wizardDescriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, errorMsg);
        return valid;
    }
    
    /**
     * Check whether the combination of issuing agency and registration 
     * number is unique across all registered projects.
     * 
     * @return 
     */
    protected boolean isUniqueEnterpriseId() {
        EnterpriseIdProvider eip = new EnterpriseIdProvider() {
            @Override
            public IssuingAgency getIssuingAgency() {
                return farmDetailPanel.getFarmIssuingAgency();
            }
            @Override
            public String getEnterpriseId() {
                return farmDetailPanel.getFarmRegistrationNumber();
            }
        };
        
        // check open farm projects:
//        return EnterpriseIdProvider.ProjectFinder.find(eip) == null;
        // check registered farm projects:
        return !FarmProjectRegistry.getInstance().contains(eip);
    }
    
    @Override
    public final void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(Object settings) {
        wizardDescriptor = (WizardDescriptor) settings;

        String name = (String) wizardDescriptor.getProperty(FarmWizardIterator.PROP_PROJECT_NAME);
        if (name == null) {
            name = Bundle.Farm_default_name();
        }
        farmDetailPanel.setFarmName(name);

        IssuingAgency issuingAgency = (IssuingAgency) wizardDescriptor.getProperty(Farm.PROP_ISSUING_AGENCY);
        farmDetailPanel.setFarmIssuingAgency(issuingAgency);

        String registrationNumber = (String) wizardDescriptor.getProperty(Farm.PROP_REGISTRATIONNUMBER);
        farmDetailPanel.setFarmRegistrationNumber(registrationNumber);
    }

    @Override
    public void storeSettings(Object settings) {
        wizardDescriptor.putProperty(Farm.PROP_ISSUING_AGENCY, farmDetailPanel.getFarmIssuingAgency());
        wizardDescriptor.putProperty(Farm.PROP_REGISTRATIONNUMBER, farmDetailPanel.getFarmRegistrationNumber());
        wizardDescriptor.putProperty(Farm.PROP_ADDRESS, farmDetailPanel.getAddress());
    }

    @Override
    public boolean isFinishPanel() {
        return true;
    }
}
